#!/usr/bin/env node --no-warnings

const fs = require("fs");
const readline = require("readline");
const stream = require("stream");
const path = require("path");

const DEFAULT_LDJSON_FILE = path.resolve(
    process.cwd(),
    "./exported.jetlag.ldjson"
);
const DEFAULT_LABEL_NAME = "";
const TOTAL_AGGREGATED_LABEL_NAME = "Total";

const getLabelResults = (items, labelName) => {
    const len = items.length;
    const average = items.reduce((a, b) => a + b) / len;

    const p90index = len * 0.9;
    const p90roundUp = items[Math.ceil(p90index)];
    const p90roundDown = items[Math.floor(p90index)];
    const p90interpolation = (p90roundUp - p90roundDown) * (p90index % 1);

    return {
        label: labelName || TOTAL_AGGREGATED_LABEL_NAME,
        samples: len,
        average: parseFloat(average.toFixed(2)),
        median: items[Math.ceil(len / 2)],
        "90%-floor": p90roundDown,
        "90%-interpolation":
            parseFloat(p90interpolation.toFixed(2)) + p90roundDown,
        "90%-ceil": p90roundUp,
        "95%": items[Math.floor(len * 0.95)],
        "99%": items[Math.floor(len * 0.99)],
        minimum: items[0],
        maximum: items[len - 1]
    };
};

const getSortedResponseTimes = items => {
    return Object.entries(items)
        .map(([key, value]) => {
            return new Array(value).fill(key);
        })
        .flat()
        .map(i => parseFloat(i) * 1000)
        .sort((a, b) => a - b);
};

const readLdjson = async (ldjson, labelName = "") => {
    const input = fs.createReadStream(ldjson);
    const rl = readline.createInterface({
        input: input,
        output: new stream()
    });

    const all = {};

    for await (const line of rl) {
        if (!line || line === "null") {
            // reached EOF
            continue;
        }
        const parsed = JSON.parse(line);
        if (!parsed.current) {
            throw new Error("Malformed entry " + line);
        }
        if (!parsed.current[labelName]) {
            // line doesn't contain label, very possible
            continue;
        }
        const rt = parsed.current[labelName].rt || {};

        for (const [key, value] of Object.entries(rt)) {
            if (!all[key]) {
                all[key] = 0;
            }
            all[key] += value;
        }
    }

    if (Object.keys(all).length === 0) {
        throw new Error(`Label "${labelName}" was not found in file`);
    }

    return all;
};

const run = async (file, labelName) => {
    let rts;
    try {
        rts = await readLdjson(file, labelName);
    } catch (e) {
        console.error(e);
        process.exit(1);
    }
    const items = getSortedResponseTimes(rts);
    const results = getLabelResults(items, labelName);

    console.table(results);
};

if (process.argv[2] === "--help" || process.argv[2] === "-h") {
    console.log(`Usage ${process.argv[1]} <ldjson_path> <label_name>`);
    console.log("Notice that the label name is case-sensitive");
    process.exit(0);
}

const ldjson = process.argv[2] || DEFAULT_LDJSON_FILE;
const labelName = process.argv[3] || DEFAULT_LABEL_NAME;

// noinspection JSIgnoredPromiseFromCall
run(ldjson, labelName);
